@@ -3,7 +3,7 @@ import sys
 
 from colorama import Fore
 
-__submodules__ = ["pipenv", "xvfb", "utils", "script"]
+__submodules__ = ["pipenv", "xvfb", "script", "boost", "tar"]
 
 __ignore__ = ["T"]
 
@@ -14,10 +14,9 @@ from snakeboost.pipenv import (
 from snakeboost.xvfb import (
     XvfbRun,
 )
-from snakeboost.utils import (
-    pipe,
-)
 from snakeboost.script import (
+    ArgAlias,
+    ArgAliasGroup,
     ParseError,
     Pyscript,
     PyscriptParam,
@@ -27,8 +26,18 @@ from snakeboost.script import (
     snakemake_args,
     snakemake_parser,
 )
+from snakeboost.boost import (
+    Boost,
+    sh_strict,
+)
+from snakeboost.tar import (
+    Tar,
+)
 
 __all__ = [
+    "ArgAlias",
+    "ArgAliasGroup",
+    "Boost",
     "ParseError",
     "PipEnv",
     "Pyscript",
@@ -36,8 +45,9 @@ __all__ = [
     "ScriptDict",
     "SnakemakeArgs",
     "SnakemakeSequenceArg",
+    "Tar",
     "XvfbRun",
-    "pipe",
+    "sh_strict",
     "snakemake_args",
     "snakemake_parser",
 ]
@@ -46,14 +56,14 @@ __all__ = [
 
 # The Tar module is dependent on python 3.8, so we restrict this module specifically
 if sys.version_info.major >= 3 and sys.version_info.minor >= 8:
-    from snakeboost.tar import (  # noqa: F401
-        Tar,
+    from snakeboost.datalad import (  # noqa: F401
+        Datalad,
     )
 
-    __all__.append("Tar")
+    __all__.append("Datalad")
 else:
     print(
         f"{Fore.YELLOW}[WARNING]: Snakeboost has only limited support for Python 3.7. "
-        "In particular, the tar module cannot be used. Please upgrade to python 3.8 or "
-        f"higher for full functionality.{Fore.RESET}"
+        "In particular, the datalad module cannot be used. Please upgrade to python "
+        f"3.8 or higher for full functionality.{Fore.RESET}"
     )
