@@ -1,10 +1,13 @@
 from __future__ import absolute_import
 
-import hashlib
 import itertools as it
 from pathlib import Path
 from typing import Iterable, List, Optional, Union
 
+from snakeboost.bash.cmd import echo, mkdir
+from snakeboost.bash.statement import Flock, ShBlock, ShIf, ShTry
+from snakeboost.utils import get_hash
+
 __all__ = ["PipEnv"]
 
 
@@ -12,11 +15,6 @@ PYTHON_VENV_CREATE_ERR = "[ERROR] (jobid={jobid}): Error creating python environ
 TIMED_OUT_ERR = "[ERROR] (jobid={jobid}): Script timed out when waiting for Python"
 
 
-def _get_hash(items: Iterable[Iterable[str]]):
-    encoded = str(sorted(*it.chain(items))).encode("utf-8")
-    return hashlib.md5(encoded).hexdigest()
-
-
 def _get_file_contents(paths: Iterable[Path]):
     for path in paths:
         with path.open("r", encoding="utf-8") as file:
@@ -36,27 +34,23 @@ class PipEnv:
     Supports thread-safe installation, so multiple jobs depending on the same venv may
     be run simultaneously.
 
-    Parameters
-    ----------
-    root : Path
-        The directory in which to place the virtualenv. Intended to be a temporary
-        directory
-    flags : str
-        Flags to include on every call of `pip install` (e.g. custom wheelhouse paths)
-    packages : List[str]
-        List of packages to install. Can be any valid pip package identifier (with or
-        without version specification)
-    requirements : List[str]
-        List of paths to requirements.txt files
+    Parameters:
+        root (Path or str): The directory in which to place the virtualenv. Intended to
+            be a temporary directory
+        flags (str):
+            Flags to include on every call of `pip install` (e.g. custom wheelhouse
+            paths)
+        packages (List[str]): List of packages to install. Can be any valid pip package
+            identifier (with or without version specification)
+        requirements (List[str]): List of paths to requirements.txt files
 
-    Attributes
-    ----------
-    venv : str
-        Path to the venv dir
-    bin : str
-        Path to the venv bin dir (e.g. venv/bin)
-    python_path : str
-        Path of the python executable (e.g. venv/bin/python)
+    Attributes:
+        venv (str):
+            Path to the venv dir
+        bin (str):
+            Path to the venv bin dir (e.g. venv/bin)
+        python_path (str):
+            Path of the python executable (e.g. venv/bin/python)
     """
 
     def __init__(
@@ -75,7 +69,9 @@ class PipEnv:
         self._dir = (
             Path(root)
             / "__snakemake_venvs__"
-            / _get_hash(filter(None, [packages, requirement_contents]))
+            / get_hash(
+                str(sorted(*it.chain(filter(None, [packages, requirement_contents]))))
+            )
         )
         self.venv = self._dir / "venv"
         self.bin = self.venv / "bin"
@@ -91,13 +87,11 @@ class PipEnv:
         This can be embedded at the beginning of a shell script to ensure the existance
         of the venv.
 
-        Typically, this should NOT be used. The class methods provide the same
-        functionality and are safer.
+        Typically, this should NOT be used. Prefer the use of :meth:`.make_venv`, or
+        any of the other methods of :class:`PipEnv`
 
-        Returns
-        -------
-        str
-            Bash script to look for a venv and create one if necessary
+        Returns:
+            str: Bash script to look for a venv and create one if necessary
         """
         install_prefix = f"{self.python_path} -m pip install {self._flags}"
         install_cmd = " && ".join(
@@ -112,17 +106,23 @@ class PipEnv:
                 ],
             )
         )
-        return (
-            f"mkdir -p {self._dir} && ("
-            f" echo '[[ -x {self.python_path} ]] ||"
-            "("
-            f"virtualenv --no-download {self.venv} && "
-            f"{install_cmd}"
-            ") || ("
-            f"echo '\"'\"'{PYTHON_VENV_CREATE_ERR}'\"'\"' 1>&2 && false"
-            f")' | flock -w 900 {self._dir} bash "
-            ")"
-        )
+        # fmt: off
+        return ShBlock(
+            mkdir(self._dir).p,
+            Flock(self._dir, wait=900).do(
+                ShIf.isnt().executable(self.python_path).then(
+                    ShTry(
+                        f"virtualenv --no-download {self.venv}",
+                        install_cmd
+                    ).catch(
+                        echo(f"{PYTHON_VENV_CREATE_ERR} 1>&2"),
+                        f"rm -rf {self.venv}",
+                        "false"
+                    )
+                )
+            ),
+        ).to_str()
+        # fmt: on
 
     def python(self, cmd: str):
         """Ensure existance of venv then run python command
@@ -134,15 +134,12 @@ class PipEnv:
         When using multiple enhancers, this must ALWAYS be the last one before the
         command.
 
-        Parameters
-        ----------
-        cmd : str
-            Command to run
+        Parameters:
+            cmd (str):
+                Command to run
 
-        Returns
-        -------
-        str
-            Modified shell script
+        Returns:
+            str: Modified shell script
         """
         return f"{self.get_venv} && {self.python_path} {cmd}"
 
@@ -156,30 +153,28 @@ class PipEnv:
         When using multiple enhancers, this must ALWAYS be the last one before the
         command.
 
-        Parameters
-        ----------
-        cmd : str
-            Command to run
+        Parameters:
+            cmd (str):
+                Command to run
 
-        Returns
-        -------
-        str
-            Modified shell script
+        Returns:
+            str: Modified shell script
         """
         stripped = cmd.strip()
-        return self.python(f"{self.venv}/bin/{stripped}")
+        return self.make_venv(f"{self.venv}/bin/{stripped}")
 
     def make_venv(self, cmd: str):
         """Ensure of existence of venv and run any arbitrary command
 
-        Parameters
-        ----------
-        cmd : str
-            Command to run
+        Parameters:
+            cmd:
+                Command to run
 
-        Returns
-        -------
-        str
-            Modified shell script
+        Returns:
+            str: Modified shell script
         """
         return f"{self.get_venv} && {cmd}"
+
+
+if __name__ == "__main__":
+    print(PipEnv("/tmp", packages=["black", "flake8"]).get_venv)
